import aiohttp
import asyncio
import json,codern

class Client:
    def __init__(self, merchant_id):
        self.merchant_id = merchant_id
        self.base_url = 'https://api.zarinpal.com/pg/v4/payman'
        self.payment_url = "https://payment.zarinpal.com/pg/v4/payment"

    async def send_request(self, session, endpoint, data):
        """
        ارسال درخواست به API زرین‌پال.

        :param session: شیء جلسه (session) از aiohttp.
        :param endpoint: نقطه‌اتصال (endpoint) که درخواست به آن ارسال می‌شود.
        :param data: داده‌هایی که باید ارسال شوند.
        :return: پاسخ JSON دریافتی از API زرین‌پال.
        """
        url = f'{self.base_url}/{endpoint}.json'
        headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
        async with session.post(url, headers=headers, data=json.dumps(data)) as response:
            return await response.json()

    async def request_payment(self, session, mobile, ssn=None, expire_at=None, max_daily_count=None, max_monthly_count=None, max_amount=None, callback_url=None):
        """
        ارسال درخواست پرداخت به زرین‌پال.

        :param session: شیء جلسه (session) از aiohttp.
        :param mobile: شماره موبایل مشتری برای ارسال پیامک.
        :param ssn: کد ملی (اختیاری).
        :param expire_at: تاریخ انقضای پرداخت (اختیاری).
        :param max_daily_count: حداکثر تعداد تراکنش‌های روزانه (اختیاری).
        :param max_monthly_count: حداکثر تعداد تراکنش‌های ماهانه (اختیاری).
        :param max_amount: حداکثر مبلغ (اختیاری).
        :param callback_url: آدرس بازگشتی پس از پرداخت.
        :return: پاسخ JSON از درخواست پرداخت.
        """
        data = {
            "merchant_id": self.merchant_id,
            "mobile": mobile,
            "ssn": ssn,
            "expire_at": expire_at,
            "max_daily_count": max_daily_count,
            "max_monthly_count": max_monthly_count,
            "max_amount": max_amount,
            "callback_url": callback_url
        }
        return await self.send_request(session, 'request', data)

    async def get_banks(self, session):
        """
        دریافت لیست بانک‌های پشتیبانی‌شده توسط زرین‌پال.

        :param session: شیء جلسه (session) از aiohttp.
        :return: لیست بانک‌ها.
        """
        url = f'{self.base_url}/banksList.json'
        async with session.get(url) as response:
            return await response.json()

    async def verify_payment(self, session, payman_authority):
        """
        تایید پرداخت پس از بازگشت از درگاه زرین‌پال.

        :param session: شیء جلسه (session) از aiohttp.
        :param payman_authority: کد شناسایی پرداخت که از درگاه زرین‌پال دریافت شده است.
        :return: پاسخ JSON از تایید پرداخت.
        """
        data = {
            "merchant_id": self.merchant_id,
            "payman_authority": payman_authority
        }
        return await self.send_request(session, 'verify', data)

    async def checkout_payment(self, session, authority, signature):
        """
        تکمیل پرداخت پس از تایید موفقیت‌آمیز.

        :param session: شیء جلسه (session) از aiohttp.
        :param authority: کد شناسایی پرداخت.
        :param signature: امضای تراکنش.
        :return: پاسخ JSON از تکمیل پرداخت.
        """
        data = {
            "merchant_id": self.merchant_id,
            "authority": authority,
            "signature": signature
        }
        return await self.send_request(session, 'checkout', data)

    async def cancel_contract(self, session, signature):
        """
        لغو قرارداد در صورت نیاز.

        :param session: شیء جلسه (session) از aiohttp.
        :param signature: امضای تراکنش.
        :return: پاسخ JSON از لغو قرارداد.
        """
        data = {
            "merchant_id": self.merchant_id,
            "signature": signature
        }
        return await self.send_request(session, 'cancelContract', data)

    async def create_payment_request(self, session, amount, description, callback_url, mobile=None, email=None, order_id=None):
        """
        ایجاد درخواست پرداخت.

        :param session: شیء جلسه (session) از aiohttp.
        :param amount: مبلغ پرداخت.
        :param description: توضیحات تراکنش.
        :param callback_url: آدرس بازگشتی پس از پرداخت.
        :param mobile: شماره موبایل (اختیاری).
        :param email: ایمیل (اختیاری).
        :param order_id: شناسه سفارش (اختیاری).
        :return: پاسخ JSON از درخواست پرداخت.
        """
        metadata = {}
        if mobile:
            metadata["mobile"] = mobile
        if email:
            metadata["email"] = email
        if order_id:
            metadata["order_id"] = order_id
        
        data = {
            "merchant_id": self.merchant_id,
            "amount": amount,
            "callback_url": callback_url,
            "description": description,
            "metadata": metadata
        }
        async with session.post(f'{self.payment_url}/request.json', json=data) as response:
            return await response.json()

    async def verify_payment_request(self, session, authority):
        """
        تایید درخواست پرداخت پس از برگشت از درگاه زرین‌پال.

        :param session: شیء جلسه (session) از aiohttp.
        :param authority: کد شناسایی درخواست پرداخت.
        :return: پاسخ JSON از تایید درخواست پرداخت.
        """
        data = {
            "merchant_id": self.merchant_id,
            "authority": authority
        }
        async with session.post(f'{self.payment_url}/verify.json', json=data) as response:
            return await response.json()